import java.io.IOException;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;
import java.util.Scanner;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;
import org.jsoup.select.Elements;

import gvjava.org.json.JSONException;
import gvjava.org.json.JSONObject;

import com.techventus.server.voice.Voice;

public class app {

	public static void main(String[] args) {
		try {
			// Read user credentials
			System.out.print("Your Google login: ");
			Scanner in = new Scanner(System.in);
			String login = in.next();
			System.out.print("Your Google password: ");
			String password = in.next();
			
			// Logging in			
			Voice voice = new Voice(login, password);
			
			// Get page and parse its <script> sections to find phones
			String page = voice.getGeneral();
			Document doc = Jsoup.parse(page);
			Elements scripts = doc.getElementsByTag("script");
			String gcData = null;
			for (Iterator<Element> iterator = scripts.iterator(); iterator.hasNext();) {
				Element e = iterator.next();
				if (e.data().contains("var _gcData")) {
					// Found phones section
					gcData = e.data();
					break;
				}
			}
			if (gcData == null) {
				System.out.println("Can't find _gcData. Something wrong with Google Voice page.");
				System.exit(1);
			}
			
			// Get phones JSON array from _gcData
			Matcher m = Pattern.compile("'phones': (.*),\n").matcher(gcData);
			if (!m.find()) {
				System.out.println("Can't find phones section in _gcData. Something wrong with Google Voice page");
				System.exit(1);
			}
			String jsonPhonesRaw = m.group(1);
			// Parse phones JSON array
			try {
				JSONObject jsonPhones = new JSONObject(jsonPhonesRaw);
				System.out.println("\nForwarding phones:");
				for (Iterator<?> iterator = jsonPhones.sortedKeys(); iterator.hasNext();) {
					JSONObject jsonPhone = jsonPhones.getJSONObject((String)iterator.next());
					int phoneId = jsonPhone.getInt("id");
					String phoneNumber = jsonPhone.getString("phoneNumber");
					System.out.printf("#%d - %s\n", phoneId, phoneNumber);
				}
				// Ask user which forwarding number to disable and disabling it
				System.out.print("Phone # to remove (Ctrl+D to cancel): ");
				if (in.hasNextInt())
					voice.phoneRemove(in.nextInt());
			} catch (JSONException e) {
				e.printStackTrace();
			}	
		} catch (IOException e) {
			e.printStackTrace();
		}
		
	}

}
